@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -20,10 +20,10 @@
  ***************************************/
 
 
-#include <sys/types.h>
 #include <stdlib.h>
 #include <math.h>
 
+#include "types.h"
 #include "nodes.h"
 #include "segments.h"
 #include "ways.h"
@@ -49,6 +49,7 @@ Nodes *LoadNodeList(const char *filename)
 {
  Nodes *nodes;
 #if SLIM
+ size_t sizeoffsets;
  int i;
 #endif
 
@@ -75,7 +76,13 @@ Nodes *LoadNodeList(const char *filename)
 
  ReadFile(nodes->fd,&nodes->file,sizeof(NodesFile));
 
- nodes->nodesoffset=sizeof(NodesFile)+(nodes->file.latbins*nodes->file.lonbins+1)*sizeof(index_t);
+ sizeoffsets=(nodes->file.latbins*nodes->file.lonbins+1)*sizeof(index_t);
+
+ nodes->offsets=(index_t*)malloc(sizeoffsets);
+
+ ReadFile(nodes->fd,nodes->offsets,sizeoffsets);
+
+ nodes->nodesoffset=sizeof(NodesFile)+sizeoffsets;
 
  for(i=0;i<sizeof(nodes->cached)/sizeof(nodes->cached[0]);i++)
     nodes->incache[i]=NO_NODE;
@@ -187,7 +194,7 @@ index_t FindClosestNode(Nodes *nodes,Segments *segments,Ways *ways,double latitu
 
           for(i=index1;i<index2;i++)
             {
-             Node *node=LookupNode(nodes,i,1);
+             Node *node=LookupNode(nodes,i,3);
              double lat=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb)+off_to_latlong(node->latoffset));
              double lon=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb)+off_to_latlong(node->lonoffset));
 
@@ -199,7 +206,7 @@ index_t FindClosestNode(Nodes *nodes,Segments *segments,Ways *ways,double latitu
 
                 /* Check that at least one segment is valid for the profile */
 
-                segment=FirstSegment(segments,nodes,i,1);
+                segment=FirstSegment(segments,node,1);
 
                 do
                   {
@@ -343,7 +350,7 @@ index_t FindClosestSegment(Nodes *nodes,Segments *segments,Ways *ways,double lat
 
           for(i=index1;i<index2;i++)
             {
-             Node *node=LookupNode(nodes,i,1);
+             Node *node=LookupNode(nodes,i,3);
              double lat1=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb)+off_to_latlong(node->latoffset));
              double lon1=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb)+off_to_latlong(node->lonoffset));
              distance_t dist1;
@@ -356,7 +363,7 @@ index_t FindClosestSegment(Nodes *nodes,Segments *segments,Ways *ways,double lat
 
                 /* Check each segment for closeness and if valid for the profile */
 
-                segment=FirstSegment(segments,nodes,i,1);
+                segment=FirstSegment(segments,node,1);
 
                 do
                   {
@@ -510,7 +517,7 @@ static int valid_segment_for_profile(Ways *ways,Segment *segment,Profile *profil
 
 void GetLatLong(Nodes *nodes,index_t index,double *latitude,double *longitude)
 {
- Node *node=LookupNode(nodes,index,2);
+ Node *node=LookupNode(nodes,index,4);
  ll_bin_t latbin=-1,lonbin=-1;
  ll_bin_t start,end,mid;
  index_t offset;
