@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <sys/time.h>
 
 #include "types.h"
 #include "ways.h"
@@ -34,11 +35,12 @@
 #include "waysx.h"
 #include "relationsx.h"
 #include "superx.h"
+#include "prunex.h"
 
 #include "files.h"
 #include "logging.h"
 #include "functions.h"
-#include "functionsx.h"
+#include "osmparser.h"
 #include "tagging.h"
 
 
@@ -50,6 +52,9 @@ char *option_tmpdirname=NULL;
 /*+ The amount of RAM to use for filesorting. +*/
 size_t option_filesort_ramsize=0;
 
+/*+ The number of threads to use for filesorting. +*/
+int option_filesort_threads=1;
+
 
 /* Local functions */
 
@@ -62,6 +67,7 @@ static void print_usage(int detail,const char *argerr,const char *err);
 
 int main(int argc,char** argv)
 {
+ struct timeval start_time;
  NodesX     *Nodes;
  SegmentsX  *Segments,*SuperSegments=NULL,*MergedSegments=NULL;
  WaysX      *Ways;
@@ -71,8 +77,11 @@ int main(int argc,char** argv)
  char       *dirname=NULL,*prefix=NULL,*tagging=NULL,*errorlog=NULL;
  int         option_parse_only=0,option_process_only=0;
  int         option_filenames=0;
+ int         option_prune_isolated=500,option_prune_short=5,option_prune_straight=3;
  int         arg;
 
+ gettimeofday(&start_time,NULL);
+
  /* Parse the command line arguments */
 
  for(arg=1;arg<argc;arg++)
@@ -81,6 +90,10 @@ int main(int argc,char** argv)
        print_usage(1,NULL,NULL);
     else if(!strncmp(argv[arg],"--sort-ram-size=",16))
        option_filesort_ramsize=atoi(&argv[arg][16]);
+#if defined(USE_PTHREADS) && USE_PTHREADS
+    else if(!strncmp(argv[arg],"--sort-threads=",15))
+       option_filesort_threads=atoi(&argv[arg][15]);
+#endif
     else if(!strncmp(argv[arg],"--dir=",6))
        dirname=&argv[arg][6];
     else if(!strncmp(argv[arg],"--tmpdir=",9))
@@ -93,6 +106,8 @@ int main(int argc,char** argv)
        option_process_only=1;
     else if(!strcmp(argv[arg],"--loggable"))
        option_loggable=1;
+    else if(!strcmp(argv[arg],"--logtime"))
+       option_logtime=1;
     else if(!strcmp(argv[arg],"--errorlog"))
        errorlog="error.log";
     else if(!strncmp(argv[arg],"--errorlog=",11))
@@ -101,6 +116,19 @@ int main(int argc,char** argv)
        max_iterations=atoi(&argv[arg][17]);
     else if(!strncmp(argv[arg],"--tagging=",10))
        tagging=&argv[arg][10];
+    else if(!strncmp(argv[arg],"--prune",7))
+      {
+       if(!strcmp(&argv[arg][7],"-none"))
+          option_prune_isolated=option_prune_short=option_prune_straight=0;
+       else if(!strncmp(&argv[arg][7],"-isolated=",10))
+          option_prune_isolated=atoi(&argv[arg][17]);
+       else if(!strncmp(&argv[arg][7],"-short=",7))
+          option_prune_short=atoi(&argv[arg][14]);
+       else if(!strncmp(&argv[arg][7],"-straight=",10))
+          option_prune_straight=atoi(&argv[arg][17]);
+       else
+          print_usage(0,argv[arg],NULL);
+      }
     else if(argv[arg][0]=='-' && argv[arg][1]=='-')
        print_usage(0,argv[arg],NULL);
     else
@@ -118,9 +146,9 @@ int main(int argc,char** argv)
  if(!option_filesort_ramsize)
    {
 #if SLIM
-       option_filesort_ramsize=64*1024*1024;
+    option_filesort_ramsize=64*1024*1024;
 #else
-       option_filesort_ramsize=256*1024*1024;
+    option_filesort_ramsize=256*1024*1024;
 #endif
    }
  else
@@ -134,33 +162,37 @@ int main(int argc,char** argv)
        option_tmpdirname=dirname;
    }
 
- if(tagging)
+ 
+ if(!option_process_only)
    {
-    if(!ExistsFile(tagging))
+    if(tagging)
       {
-       fprintf(stderr,"Error: The '--tagging' option specifies a file that does not exist.\n");
-       return(1);
+       if(!ExistsFile(tagging))
+         {
+          fprintf(stderr,"Error: The '--tagging' option specifies a file that does not exist.\n");
+          return(1);
+         }
       }
-   }
- else
-   {
-    if(ExistsFile(FileName(dirname,prefix,"tagging.xml")))
-       tagging=FileName(dirname,prefix,"tagging.xml");
-    else if(ExistsFile(FileName(DATADIR,NULL,"tagging.xml")))
-       tagging=FileName(DATADIR,NULL,"tagging.xml");
     else
       {
-       fprintf(stderr,"Error: The '--tagging' option was not used and the default 'tagging.xml' does not exist.\n");
+       if(ExistsFile(FileName(dirname,prefix,"tagging.xml")))
+          tagging=FileName(dirname,prefix,"tagging.xml");
+       else if(ExistsFile(FileName(DATADIR,NULL,"tagging.xml")))
+          tagging=FileName(DATADIR,NULL,"tagging.xml");
+       else
+         {
+          fprintf(stderr,"Error: The '--tagging' option was not used and the default 'tagging.xml' does not exist.\n");
+          return(1);
+         }
+      }
+
+    if(ParseXMLTaggingRules(tagging))
+      {
+       fprintf(stderr,"Error: Cannot read the tagging rules in the file '%s'.\n",tagging);
        return(1);
       }
    }
 
- if(ParseXMLTaggingRules(tagging))
-   {
-    fprintf(stderr,"Error: Cannot read the tagging rules in the file '%s'.\n",tagging);
-    return(1);
-   }
-
  /* Create new node, segment, way and relation variables */
 
  Nodes=NewNodeList(option_parse_only||option_process_only);
@@ -178,40 +210,45 @@ int main(int argc,char** argv)
 
  /* Parse the file */
 
- if(option_filenames)
-   {
-    for(arg=1;arg<argc;arg++)
-      {
-       FILE *file;
+if(!option_process_only)
+  {
+   if(option_filenames)
+     {
+      for(arg=1;arg<argc;arg++)
+        {
+         FILE *file;
 
-       if(argv[arg][0]=='-' && argv[arg][1]=='-')
-          continue;
+         if(argv[arg][0]=='-' && argv[arg][1]=='-')
+            continue;
 
-       file=fopen(argv[arg],"rb");
+         file=fopen(argv[arg],"rb");
 
-       if(!file)
-         {
-          fprintf(stderr,"Cannot open file '%s' for reading [%s].\n",argv[arg],strerror(errno));
-          exit(EXIT_FAILURE);
-         }
+         if(!file)
+           {
+            fprintf(stderr,"Cannot open file '%s' for reading [%s].\n",argv[arg],strerror(errno));
+            exit(EXIT_FAILURE);
+           }
 
-       printf("\nParse OSM Data [%s]\n==============\n\n",argv[arg]);
-       fflush(stdout);
+         printf("\nParse OSM Data [%s]\n==============\n\n",argv[arg]);
+         fflush(stdout);
 
-       if(ParseOSM(file,Nodes,Segments,Ways,Relations))
-          exit(EXIT_FAILURE);
+         if(ParseOSM(file,Nodes,Segments,Ways,Relations))
+            exit(EXIT_FAILURE);
 
-       fclose(file);
-      }
-   }
- else if(!option_process_only)
-   {
-    printf("\nParse OSM Data\n==============\n\n");
-    fflush(stdout);
+         fclose(file);
+        }
+     }
+   else
+     {
+      printf("\nParse OSM Data\n==============\n\n");
+      fflush(stdout);
 
-    if(ParseOSM(stdin,Nodes,Segments,Ways,Relations))
-       exit(EXIT_FAILURE);
-   }
+      if(ParseOSM(stdin,Nodes,Segments,Ways,Relations))
+         exit(EXIT_FAILURE);
+     }
+
+   DeleteXMLTaggingRules();
+  }
 
  if(option_parse_only)
    {
@@ -223,8 +260,6 @@ int main(int argc,char** argv)
     return(0);
    }
 
- DeleteXMLTaggingRules();
-
  /* Process the data */
 
  printf("\nProcess OSM Data\n================\n\n");
@@ -234,7 +269,7 @@ int main(int argc,char** argv)
 
  SortNodeList(Nodes);
 
- SortSegmentList(Segments);
+ SortSegmentList(Segments,0);
 
  SortWayList(Ways);
 
@@ -270,6 +305,23 @@ int main(int argc,char** argv)
 
  ProcessTurnRelations2(Relations,Nodes,Segments,Ways);
 
+ /* Prune unwanted nodes/segments. */
+
+ if(option_prune_straight || option_prune_isolated || option_prune_short)
+   {
+    StartPruning(Nodes,Segments,Ways);
+
+    if(option_prune_straight)
+       PruneStraightHighwayNodes(Nodes,Segments,Ways,option_prune_straight);
+
+    if(option_prune_isolated)
+       PruneIsolatedRegions(Nodes,Segments,Ways,option_prune_isolated);
+
+    if(option_prune_short)
+       PruneShortSegments(Nodes,Segments,Ways,option_prune_short);
+
+    FinishPruning(Nodes,Segments,Ways);
+   }
 
  /* Repeated iteration on Super-Nodes and Super-Segments */
 
@@ -313,7 +365,7 @@ int main(int argc,char** argv)
 
     /* Sort the super-segments */
 
-    SortSegmentList(SuperSegments);
+    SortSegmentList(SuperSegments,0);
 
     /* Remove duplicated super-segments */
 
@@ -352,7 +404,7 @@ int main(int argc,char** argv)
 
  /* Sort and re-index the segments */
 
- SortSegmentList(Segments);
+ SortSegmentList(Segments,0);
 
  IndexSegments(Segments,Nodes);
 
@@ -369,7 +421,7 @@ int main(int argc,char** argv)
 
  /* Sort the segments geographically and re-index them */
 
- SortSegmentList(Segments);
+ SortSegmentList(Segments,0);
 
  IndexSegments(Segments,Nodes);
 
@@ -417,6 +469,16 @@ int main(int argc,char** argv)
  if(errorlog)
     close_errorlog();
 
+ /* Print the total time */
+
+ if(option_logtime)
+   {
+    printf("\n");
+    fprintf_elapsed_time(stdout,&start_time);
+    printf("Complete\n");
+    fflush(stdout);
+   }
+
  return(0);
 }
 
@@ -436,12 +498,21 @@ static void print_usage(int detail,const char *argerr,const char *err)
  fprintf(stderr,
          "Usage: planetsplitter [--help]\n"
          "                      [--dir=<dirname>] [--prefix=<name>]\n"
+#if defined(USE_PTHREADS) && USE_PTHREADS
+         "                      [--sort-ram-size=<size>] [--sort-threads=<number>]\n"
+#else
          "                      [--sort-ram-size=<size>]\n"
+#endif
          "                      [--tmpdir=<dirname>]\n"
-         "                      [--parse-only | --process-only]\n"
-         "                      [--loggable] [--errorlog[=<name>]]\n"
-         "                      [--max-iterations=<number>]\n"
          "                      [--tagging=<filename>]\n"
+         "                      [--loggable] [--logtime]\n"
+         "                      [--errorlog[=<name>]]\n"
+         "                      [--parse-only | --process-only]\n"
+         "                      [--max-iterations=<number>]\n"
+         "                      [--prune-none]\n"
+         "                      [--prune-isolated=<len>]\n"
+         "                      [--prune-short=<len>]\n"
+         "                      [--prune-straight=<len>]\n"
          "                      [<filename.osm> ...]\n");
 
  if(argerr)
@@ -468,24 +539,38 @@ static void print_usage(int detail,const char *argerr,const char *err)
 #else
             "                          (defaults to 256MB otherwise.)\n"
 #endif
+#if defined(USE_PTHREADS) && USE_PTHREADS
+            "--sort-threads=<number>   The number of threads to use for data sorting.\n"
+#endif
+            "\n"
             "--tmpdir=<dirname>        The directory name for temporary files.\n"
             "                          (defaults to the '--dir' option directory.)\n"
             "\n"
-            "--parse-only              Parse the input OSM files and store the results.\n"
-            "--process-only            Process the stored results from previous option.\n"
-            "\n"
-            "--loggable                Print progress messages suitable for logging to file.\n"
-            "--errorlog[=<name>]       Log parsing errors to 'error.log' or the given name\n"
-            "                          (the '--dir' and '--prefix' options are applied).\n"
-            "\n"
-            "--max-iterations=<number> The number of iterations for finding super-nodes\n"
-            "                          (defaults to 5).\n"
-            "\n"
             "--tagging=<filename>      The name of the XML file containing the tagging rules\n"
             "                          (defaults to 'tagging.xml' with '--dir' and\n"
             "                           '--prefix' options or the file installed in\n"
             "                           '" DATADIR "').\n"
             "\n"
+            "--loggable                Print progress messages suitable for logging to file.\n"
+            "--logtime                 Print the elapsed time for each processing step.\n"
+            "--errorlog[=<name>]       Log parsing errors to 'error.log' or the given name\n"
+            "                          (the '--dir' and '--prefix' options are applied).\n"
+            "\n"
+            "--parse-only              Parse the input OSM files and store the results.\n"
+            "--process-only            Process the stored results from previous option.\n"
+            "\n"
+            "--max-iterations=<number> The number of iterations for finding super-nodes\n"
+            "                          (defaults to 5).\n"
+            "\n"
+            "--prune-none              Disable the prune options below, they are re-enabled\n"
+            "                          by adding them to the command line after this option.\n"
+            "--prune-isolated=<len>    Remove small disconnected groups of segments\n"
+            "                          (defaults to removing groups under 500m).\n"
+            "--prune-short=<len>       Remove short segments (defaults to removing segments\n"
+            "                          up to a maximum length of 5m).\n"
+            "--prune-straight=<len>    Remove nodes in almost straight highways (defaults to\n"
+            "                          removing nodes up to 3m offset from a straight line).\n"
+            "\n"
             "<filename.osm> ...        The name(s) of the file(s) to process (by default\n"
             "                          data is read from standard input).\n"
             "\n"
