@@ -201,7 +201,7 @@ void AppendRouteRelation(RelationsX* relationsx,relation_t id,
 
  relationsx->rnumber++;
 
- assert(!(relationsx->rnumber==0)); /* Zero marks the high-water mark for relations. */
+ assert(relationsx->rnumber!=0); /* Zero marks the high-water mark for relations. */
 }
 
 
@@ -240,7 +240,7 @@ void AppendTurnRestrictRelation(RelationsX* relationsx,relation_t id,
 
  relationsx->trnumber++;
 
- assert(!(relationsx->trnumber==0)); /* Zero marks the high-water mark for relations. */
+ assert(relationsx->trnumber!=0); /* Zero marks the high-water mark for relations. */
 }
 
 
@@ -537,13 +537,21 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
 
                 if(routes&Transports_Foot)
                   {
-                   wayx->way.allow|=Transports_Foot;
+                   if(!(wayx->way.allow&Transports_Foot))
+                     {
+                      logerror("Route Relation %"Prelation_t" for Foot contains Way %"Pway_t" that does not allow Foot transport; overriding.\n",relationx.id,wayid);
+                      wayx->way.allow|=Transports_Foot;
+                     }
                    wayx->way.props|=Properties_FootRoute;
                   }
 
                 if(routes&Transports_Bicycle)
                   {
-                   wayx->way.allow|=Transports_Bicycle;
+                   if(!(wayx->way.allow&Transports_Bicycle))
+                     {
+                      logerror("Route Relation %"Prelation_t" for Bicycle contains Way %"Pway_t" that does not allow Bicycle transport; overriding.\n",relationx.id,wayid);
+                      wayx->way.allow|=Transports_Bicycle;
+                     }
                    wayx->way.props|=Properties_BicycleRoute;
                   }
 
