@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -22,10 +22,12 @@
 
 #include <assert.h>
 #include <stdlib.h>
-#include <stdio.h>
 
+#include "types.h"
+#include "segments.h"
 #include "ways.h"
 
+#include "typesx.h"
 #include "nodesx.h"
 #include "segmentsx.h"
 #include "waysx.h"
@@ -63,6 +65,17 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
  printf_first("Finding Super-Nodes: Nodes=0 Super-Nodes=0");
 
+ /* Allocate and set the super-node markers */
+
+ if(!nodesx->super)
+   {
+    nodesx->super=AllocBitMask(nodesx->number);
+
+    assert(nodesx->super); /* Check AllocBitMask() worked */
+
+    SetAllBits1(nodesx->super,nodesx->number);
+   }
+
  /* Map into memory / open the files */
 
 #if !SLIM
@@ -84,7 +97,9 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
        int issuper=0;
        NodeX *nodex=LookupNodeX(nodesx,i,1);
 
-       if(nodex->flags&(NODE_TURNRSTRCT|NODE_TURNRSTRCT2))
+       if(IsPrunedNodeX(nodex))
+          issuper=0;
+       else if(nodex->flags&(NODE_TURNRSTRCT|NODE_TURNRSTRCT2))
           issuper=1;
        else
          {
@@ -109,9 +124,9 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
              segmentway[count]=wayx->way;
 
-             /* If the node allows less traffic types than any connecting way then it is super */
+             /* If the node allows less traffic types than any connecting way then it is super if it allows anything */
 
-             if((wayx->way.allow&nodex->allow)!=wayx->way.allow)
+             if((wayx->way.allow&nodex->allow)!=wayx->way.allow && nodex->allow!=Transports_None)
                {
                 issuper=1;
                 break;
@@ -143,7 +158,7 @@ void ChooseSuperNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
              if(issuper)
                 break;
 
-             segmentx=NextSegmentX(segmentsx,segmentx,i,1);
+             segmentx=NextSegmentX(segmentsx,segmentx,i);
 
              count++;
             }
@@ -209,9 +224,11 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
  /* Map into memory / open the files */
 
 #if !SLIM
+ nodesx->data=MapFile(nodesx->filename);
  segmentsx->data=MapFile(segmentsx->filename);
  waysx->data=MapFile(waysx->filename);
 #else
+ nodesx->fd=ReOpenFile(nodesx->filename);
  segmentsx->fd=ReOpenFile(segmentsx->filename);
  waysx->fd=ReOpenFile(waysx->filename);
 #endif
@@ -277,7 +294,7 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
              FreeResultsList(results);
             }
 
-          segmentx=NextSegmentX(segmentsx,segmentx,i,1);
+          segmentx=NextSegmentX(segmentsx,segmentx,i);
          }
 
        sn++;
@@ -290,9 +307,11 @@ SegmentsX *CreateSuperSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
  /* Unmap from memory / close the files */
 
 #if !SLIM
+ nodesx->data=UnmapFile(nodesx->filename);
  segmentsx->data=UnmapFile(segmentsx->filename);
  waysx->data=UnmapFile(waysx->filename);
 #else
+ nodesx->fd=CloseFile(nodesx->fd);
  segmentsx->fd=CloseFile(segmentsx->fd);
  waysx->fd=CloseFile(waysx->fd);
 #endif
@@ -435,7 +454,7 @@ static Results *FindRoutesWay(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,n
 
  /* Insert the first node into the queue */
 
- results=NewResultsList(4);
+ results=NewResultsList(64);
 
  queue=NewQueueList();
 
@@ -456,6 +475,7 @@ static Results *FindRoutesWay(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,n
 
     while(segmentx)
       {
+       NodeX  *node2x;
        index_t node2,seg2;
        distance_t cumulative_distance;
 
@@ -463,20 +483,26 @@ static Results *FindRoutesWay(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,n
        if(IsOnewayTo(segmentx,node1))
           goto endloop;
 
-       node2=OtherNode(segmentx,node1);
-
        seg2=IndexSegmentX(segmentsx,segmentx);
 
        /* must not be a u-turn */
        if(result1->segment==seg2)
           goto endloop;
 
-       wayx=LookupWayX(waysx,segmentx->way,2);
+       wayx=LookupWayX(waysx,segmentx->way,2); /* position 1 is already used */
 
        /* must be the right type of way */
        if(WaysCompare(&wayx->way,match))
           goto endloop;
 
+       node2=OtherNode(segmentx,node1);
+
+       node2x=LookupNodeX(nodesx,node2,2); /* position 1 is already used */
+
+       /* Don't route beyond a node with no access */
+       if(node2x->allow==Transports_None)
+          goto endloop;
+
        cumulative_distance=(distance_t)result1->score+DISTANCE(segmentx->distance);
 
        result2=FindResult(results,node2,seg2);
@@ -505,7 +531,7 @@ static Results *FindRoutesWay(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,n
 
       endloop:
 
-       segmentx=NextSegmentX(segmentsx,segmentx,node1,2);
+       segmentx=NextSegmentX(segmentsx,segmentx,node1);
       }
    }
 
