@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -23,6 +23,7 @@
 #ifndef TYPES_H
 #define TYPES_H    /*+ To stop multiple inclusions. +*/
 
+#include <inttypes.h>
 #include <stdint.h>
 #include <math.h>
 
@@ -50,6 +51,9 @@
 /*+ An undefined relation index. +*/
 #define NO_RELATION    (~(index_t)0)
 
+/*+ An undefined location. +*/
+#define NO_LATLONG     ((latlong_t)0x80000000)
+
 
 /*+ The lowest number allowed for a fake node. +*/
 #define NODE_FAKE      ((index_t)0xffff0000)
@@ -92,10 +96,10 @@
 /*+ A flag to mark a segment as a normal segment. +*/
 #define SEGMENT_NORMAL ((distance_t)0x10000000)
 
-/*+ The real distance ignoring the ONEWAY_* and SEGMENT_* flags. +*/
+/*+ The real distance ignoring the other flags. +*/
 #define DISTANCE(xx)   ((distance_t)((xx)&(~(ONEWAY_1TO2|ONEWAY_2TO1|SEGMENT_SUPER|SEGMENT_NORMAL))))
 
-/*+ The distance flags selecting only the ONEWAY_* and SEGMENT_* flags. +*/
+/*+ The distance flags selecting only the flags. +*/
 #define DISTFLAG(xx)   ((distance_t)((xx)&(ONEWAY_1TO2|ONEWAY_2TO1|SEGMENT_SUPER|SEGMENT_NORMAL)))
 
 
@@ -112,6 +116,9 @@
 /*+ A node, segment, way or relation index. +*/
 typedef uint32_t index_t;
 
+/*+ A printf formatting string for an index_t type (this should match the index_t definition above). +*/
+#define Pindex_t PRIu32         /* PRIu32 and PRIu64 are defined in intypes.h */
+
 
 /*+ A node latitude or longitude (range: +/-pi*LAT_LONG_SCALE = +/-3.14*1024*65536 = ~29 bits). +*/
 typedef int32_t  latlong_t;
@@ -205,7 +212,8 @@ typedef enum _Highway
 
   Way_Count       =14,       /* One more than the number of highway types. */
 
-  Way_OneWay      =32
+  Way_OneWay      =32,
+  Way_Roundabout  =64
  }
  Highway;
 
@@ -283,7 +291,7 @@ typedef enum _Transports
   Transports_HGV        = TRANSPORTS(Transport_HGV       ),
   Transports_PSV        = TRANSPORTS(Transport_PSV       ),
 
-  Transports_ALL        = 65535
+  Transports_ALL        = TRANSPORTS(Transport_Count     )-1
  }
  Transports;
 
@@ -318,14 +326,14 @@ typedef enum _Properties
  {
   Properties_None         = 0,
 
-  Properties_Paved        = PROPERTIES(Property_Paved),
-  Properties_Multilane    = PROPERTIES(Property_Multilane),
-  Properties_Bridge       = PROPERTIES(Property_Bridge),
-  Properties_Tunnel       = PROPERTIES(Property_Tunnel),
-  Properties_FootRoute    = PROPERTIES(Property_FootRoute),
+  Properties_Paved        = PROPERTIES(Property_Paved       ),
+  Properties_Multilane    = PROPERTIES(Property_Multilane   ),
+  Properties_Bridge       = PROPERTIES(Property_Bridge      ),
+  Properties_Tunnel       = PROPERTIES(Property_Tunnel      ),
+  Properties_FootRoute    = PROPERTIES(Property_FootRoute   ),
   Properties_BicycleRoute = PROPERTIES(Property_BicycleRoute),
 
-  Properties_ALL          = 255
+  Properties_ALL          = PROPERTIES(Property_Count       )-1
  }
  Properties;
 
