@@ -72,6 +72,7 @@ static void output_super(index_t node,double latitude,double longitude);
 static void output_oneway(index_t node,double latitude,double longitude);
 static void output_highway(index_t node,double latitude,double longitude);
 static void output_transport(index_t node,double latitude,double longitude);
+static void output_barrier(index_t node,double latitude,double longitude);
 static void output_turnrestriction(index_t node,double latitude,double longitude);
 static void output_limits(index_t node,double latitude,double longitude);
 
@@ -389,7 +390,7 @@ static void output_highway(index_t node,double latitude,double longitude)
          {
           Way *way=LookupWay(OSMWays,segment->way,1);
 
-          if(way->type==highways)
+          if(HIGHWAY(way->type)==highways)
             {
              double lat,lon;
 
@@ -494,6 +495,69 @@ static void output_transport(index_t node,double latitude,double longitude)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Output the data for nodes disallowed for a paticular type of traffic.
+
+  Nodes *nodes The set of nodes to use.
+
+  Segments *segments The set of segments to use.
+
+  Ways *ways The set of ways to use.
+
+  Relations *relations The set of relations to use.
+
+  double latmin The minimum latitude.
+
+  double latmax The maximum latitude.
+
+  double lonmin The minimum longitude.
+
+  double lonmax The maximum longitude.
+
+  Transport transport The type of transport.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void OutputBarrier(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,double latmin,double latmax,double lonmin,double lonmax,Transport transport)
+{
+ /* Use local variables so that the callback doesn't need to pass them backwards and forwards */
+
+ OSMNodes=nodes;
+ OSMSegments=segments;
+ OSMWays=ways;
+ OSMRelations=relations;
+
+ LatMin=latmin;
+ LatMax=latmax;
+ LonMin=lonmin;
+ LonMax=lonmax;
+
+ /* Iterate through the nodes and process them */
+
+ transports=TRANSPORTS(transport);
+
+ find_all_nodes(nodes,(callback_t)output_barrier);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Process a single node (called as a callback).
+
+  index_t node The node to output.
+
+  double latitude The latitude of the node.
+
+  double longitude The longitude of the node.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void output_barrier(index_t node,double latitude,double longitude)
+{
+ Node *nodep=LookupNode(OSMNodes,node,1);
+
+ if(!(nodep->allow&transports))
+    printf("%.6f %.6f\n",radians_to_degrees(latitude),radians_to_degrees(longitude));
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Output the data for turn restrictions.
 
