@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -22,12 +22,12 @@
 
 #include <assert.h>
 #include <stdlib.h>
-#include <stdio.h>
 #include <string.h>
-#include <sys/stat.h>
 
+#include "types.h"
 #include "ways.h"
 
+#include "typesx.h"
 #include "waysx.h"
 
 #include "files.h"
@@ -164,7 +164,7 @@ void AppendWay(WaysX *waysx,way_t id,Way *way,const char *name)
 
  waysx->number++;
 
- assert(!(waysx->number==0)); /* Zero marks the high-water mark for ways. */
+ assert(waysx->number!=0); /* Zero marks the high-water mark for ways. */
 }
 
 
@@ -300,7 +300,7 @@ void SortWayList(WaysX *waysx)
 
  sortwaysx=waysx;
 
- filesort_fixed(waysx->fd,fd,sizeof(WayX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))deduplicate_and_index_by_id);
+ waysx->number=filesort_fixed(waysx->fd,fd,sizeof(WayX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))deduplicate_and_index_by_id);
 
  /* Close the files */
 
@@ -522,8 +522,6 @@ static int deduplicate_and_index_by_id(WayX *wayx,index_t index)
    {
     previd=wayx->id;
 
-    sortwaysx->number++;
-
     sortwaysx->idata[index]=wayx->id;
 
     return(1);
